# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tag import Tag as Tag
from .app_id import AppID as AppID
from .bookmark import Bookmark as Bookmark
from .decision import Decision as Decision
from .access_user import AccessUser as AccessUser
from .certificate import Certificate as Certificate
from .custom_page import CustomPage as CustomPage
from .allowed_idps import AllowedIdPs as AllowedIdPs
from .cors_headers import CORSHeaders as CORSHeaders
from .oidc_saas_app import OIDCSaaSApp as OIDCSaaSApp
from .saml_saas_app import SAMLSaaSApp as SAMLSaaSApp
from .service_token import ServiceToken as ServiceToken
from .approval_group import ApprovalGroup as ApprovalGroup
from .allowed_headers import AllowedHeaders as AllowedHeaders
from .allowed_methods import AllowedMethods as AllowedMethods
from .allowed_origins import AllowedOrigins as AllowedOrigins
from .tag_list_params import TagListParams as TagListParams
from .application_type import ApplicationType as ApplicationType
from .key_get_response import KeyGetResponse as KeyGetResponse
from .user_list_params import UserListParams as UserListParams
from .zero_trust_group import ZeroTrustGroup as ZeroTrustGroup
from .group_list_params import GroupListParams as GroupListParams
from .key_update_params import KeyUpdateParams as KeyUpdateParams
from .tag_create_params import TagCreateParams as TagCreateParams
from .tag_update_params import TagUpdateParams as TagUpdateParams
from .user_get_response import UserGetResponse as UserGetResponse
from .cors_headers_param import CORSHeadersParam as CORSHeadersParam
from .group_get_response import GroupGetResponse as GroupGetResponse
from .policy_list_params import PolicyListParams as PolicyListParams
from .user_create_params import UserCreateParams as UserCreateParams
from .user_list_response import UserListResponse as UserListResponse
from .user_update_params import UserUpdateParams as UserUpdateParams
from .group_create_params import GroupCreateParams as GroupCreateParams
from .group_list_response import GroupListResponse as GroupListResponse
from .group_update_params import GroupUpdateParams as GroupUpdateParams
from .key_rotate_response import KeyRotateResponse as KeyRotateResponse
from .key_update_response import KeyUpdateResponse as KeyUpdateResponse
from .oidc_saas_app_param import OIDCSaaSAppParam as OIDCSaaSAppParam
from .policy_get_response import PolicyGetResponse as PolicyGetResponse
from .saml_saas_app_param import SAMLSaaSAppParam as SAMLSaaSAppParam
from .scim_config_mapping import SCIMConfigMapping as SCIMConfigMapping
from .self_hosted_domains import SelfHostedDomains as SelfHostedDomains
from .tag_delete_response import TagDeleteResponse as TagDeleteResponse
from .approval_group_param import ApprovalGroupParam as ApprovalGroupParam
from .associated_hostnames import AssociatedHostnames as AssociatedHostnames
from .policy_create_params import PolicyCreateParams as PolicyCreateParams
from .policy_list_response import PolicyListResponse as PolicyListResponse
from .policy_update_params import PolicyUpdateParams as PolicyUpdateParams
from .user_create_response import UserCreateResponse as UserCreateResponse
from .user_update_response import UserUpdateResponse as UserUpdateResponse
from .group_create_response import GroupCreateResponse as GroupCreateResponse
from .group_delete_response import GroupDeleteResponse as GroupDeleteResponse
from .group_update_response import GroupUpdateResponse as GroupUpdateResponse
from .bookmark_create_params import BookmarkCreateParams as BookmarkCreateParams
from .bookmark_update_params import BookmarkUpdateParams as BookmarkUpdateParams
from .policy_create_response import PolicyCreateResponse as PolicyCreateResponse
from .policy_delete_response import PolicyDeleteResponse as PolicyDeleteResponse
from .policy_update_response import PolicyUpdateResponse as PolicyUpdateResponse
from .application_list_params import ApplicationListParams as ApplicationListParams
from .certificate_list_params import CertificateListParams as CertificateListParams
from .custom_page_list_params import CustomPageListParams as CustomPageListParams
from .saas_app_name_id_format import SaaSAppNameIDFormat as SaaSAppNameIDFormat
from .application_get_response import ApplicationGetResponse as ApplicationGetResponse
from .bookmark_delete_response import BookmarkDeleteResponse as BookmarkDeleteResponse
from .custom_page_without_html import CustomPageWithoutHTML as CustomPageWithoutHTML
from .gateway_ca_list_response import GatewayCAListResponse as GatewayCAListResponse
from .application_create_params import ApplicationCreateParams as ApplicationCreateParams
from .application_list_response import ApplicationListResponse as ApplicationListResponse
from .application_update_params import ApplicationUpdateParams as ApplicationUpdateParams
from .certificate_create_params import CertificateCreateParams as CertificateCreateParams
from .certificate_update_params import CertificateUpdateParams as CertificateUpdateParams
from .custom_page_create_params import CustomPageCreateParams as CustomPageCreateParams
from .custom_page_update_params import CustomPageUpdateParams as CustomPageUpdateParams
from .scim_config_mapping_param import SCIMConfigMappingParam as SCIMConfigMappingParam
from .service_token_list_params import ServiceTokenListParams as ServiceTokenListParams
from .gateway_ca_create_response import GatewayCACreateResponse as GatewayCACreateResponse
from .gateway_ca_delete_response import GatewayCADeleteResponse as GatewayCADeleteResponse
from .application_create_response import ApplicationCreateResponse as ApplicationCreateResponse
from .application_delete_response import ApplicationDeleteResponse as ApplicationDeleteResponse
from .application_update_response import ApplicationUpdateResponse as ApplicationUpdateResponse
from .certificate_delete_response import CertificateDeleteResponse as CertificateDeleteResponse
from .custom_page_delete_response import CustomPageDeleteResponse as CustomPageDeleteResponse
from .service_token_create_params import ServiceTokenCreateParams as ServiceTokenCreateParams
from .service_token_rotate_params import ServiceTokenRotateParams as ServiceTokenRotateParams
from .service_token_update_params import ServiceTokenUpdateParams as ServiceTokenUpdateParams
from .saml_certificate_list_params import SAMLCertificateListParams as SAMLCertificateListParams
from .saml_certificate_get_response import SAMLCertificateGetResponse as SAMLCertificateGetResponse
from .service_token_create_response import ServiceTokenCreateResponse as ServiceTokenCreateResponse
from .service_token_rotate_response import ServiceTokenRotateResponse as ServiceTokenRotateResponse
from .saml_certificate_list_response import SAMLCertificateListResponse as SAMLCertificateListResponse
from .saml_certificate_rotate_response import SAMLCertificateRotateResponse as SAMLCertificateRotateResponse
from .scim_config_authentication_oauth2 import SCIMConfigAuthenticationOauth2 as SCIMConfigAuthenticationOauth2
from .scim_config_authentication_http_basic import (
    SCIMConfigAuthenticationHTTPBasic as SCIMConfigAuthenticationHTTPBasic,
)
from .scim_config_authentication_oauth2_param import (
    SCIMConfigAuthenticationOauth2Param as SCIMConfigAuthenticationOauth2Param,
)
from .scim_config_authentication_http_basic_param import (
    SCIMConfigAuthenticationHTTPBasicParam as SCIMConfigAuthenticationHTTPBasicParam,
)
from .scim_config_authentication_oauth_bearer_token import (
    SCIMConfigAuthenticationOAuthBearerToken as SCIMConfigAuthenticationOAuthBearerToken,
)
from .scim_config_authentication_oauth_bearer_token_param import (
    SCIMConfigAuthenticationOAuthBearerTokenParam as SCIMConfigAuthenticationOAuthBearerTokenParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SinkDeleteParams"]


class SinkDeleteParams(TypedDict, total=False):
    account_id: Required[str]
    """Specifies the public ID of the account."""

    force: str
    """Delete sink forcefully, including deleting any dependent pipelines."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .target import Target
from ..._utils import PropertyInfo
from ..._models import BaseModel
from ..zones.ssl import SSL
from ..zones.waf import WAF
from ..zones.mirage import Mirage
from ..zones.polish import Polish
from ..zones.cache_level import CacheLevel
from ..zones.browser_check import BrowserCheck
from ..zones.rocket_loader import RocketLoader
from ..zones.ip_geolocation import IPGeolocation
from ..zones.security_level import SecurityLevel
from ..zones.always_use_https import AlwaysUseHTTPS
from ..zones.browser_cache_ttl import BrowserCacheTTL
from ..zones.email_obfuscation import EmailObfuscation
from ..zones.response_buffering import ResponseBuffering
from ..zones.true_client_ip_header import TrueClientIPHeader
from ..zones.automatic_https_rewrites import AutomaticHTTPSRewrites
from ..zones.opportunistic_encryption import OpportunisticEncryption
from ..zones.origin_error_page_pass_thru import OriginErrorPagePassThru
from ..zones.sort_query_string_for_cache import SortQueryStringForCache

__all__ = [
    "PageRule",
    "Action",
    "ActionBypassCacheOnCookie",
    "ActionCacheByDeviceType",
    "ActionCacheDeceptionArmor",
    "ActionCacheKeyFields",
    "ActionCacheKeyFieldsValue",
    "ActionCacheKeyFieldsValueCookie",
    "ActionCacheKeyFieldsValueHeader",
    "ActionCacheKeyFieldsValueHost",
    "ActionCacheKeyFieldsValueQueryString",
    "ActionCacheKeyFieldsValueUser",
    "ActionCacheOnCookie",
    "ActionCacheTTLByStatus",
    "ActionDisableApps",
    "ActionDisablePerformance",
    "ActionDisableSecurity",
    "ActionDisableZaraz",
    "ActionEdgeCacheTTL",
    "ActionExplicitCacheControl",
    "ActionForwardingURL",
    "ActionForwardingURLValue",
    "ActionHostHeaderOverride",
    "ActionResolveOverride",
    "ActionRespectStrongEtag",
]


class ActionBypassCacheOnCookie(BaseModel):
    id: Optional[Literal["bypass_cache_on_cookie"]] = None
    """
    Bypass cache and fetch resources from the origin server if a regular expression
    matches against a cookie name present in the request.
    """

    value: Optional[str] = None
    """The regular expression to use for matching cookie names in the request.

    Refer to
    [Bypass Cache on Cookie setting](https://developers.cloudflare.com/rules/page-rules/reference/additional-reference/#bypass-cache-on-cookie-setting)
    to learn about limited regular expression support.
    """


class ActionCacheByDeviceType(BaseModel):
    id: Optional[Literal["cache_by_device_type"]] = None
    """Separate cached content based on the visitor's device type."""

    value: Optional[Literal["on", "off"]] = None
    """The status of Cache By Device Type."""


class ActionCacheDeceptionArmor(BaseModel):
    id: Optional[Literal["cache_deception_armor"]] = None
    """
    Protect from web cache deception attacks while still allowing static assets to
    be cached. This setting verifies that the URL's extension matches the returned
    `Content-Type`.
    """

    value: Optional[Literal["on", "off"]] = None
    """The status of Cache Deception Armor."""


class ActionCacheKeyFieldsValueCookie(BaseModel):
    """Controls which cookies appear in the Cache Key."""

    check_presence: Optional[List[str]] = None
    """
    A list of cookies to check for the presence of, without including their actual
    values.
    """

    include: Optional[List[str]] = None
    """A list of cookies to include."""


class ActionCacheKeyFieldsValueHeader(BaseModel):
    """Controls which headers go into the Cache Key.

    Exactly one of
    `include` or `exclude` is expected.
    """

    check_presence: Optional[List[str]] = None
    """
    A list of headers to check for the presence of, without including their actual
    values.
    """

    exclude: Optional[List[str]] = None
    """A list of headers to ignore."""

    include: Optional[List[str]] = None
    """A list of headers to include."""


class ActionCacheKeyFieldsValueHost(BaseModel):
    """Determines which host header to include in the Cache Key."""

    resolved: Optional[bool] = None
    """Whether to include the Host header in the HTTP request sent to the origin."""


class ActionCacheKeyFieldsValueQueryString(BaseModel):
    """Controls which URL query string parameters go into the Cache
    Key.

    Exactly one of `include` or `exclude` is expected.
    """

    exclude: Union[Literal["*"], List[str], None] = None
    """Ignore all query string parameters."""

    include: Union[Literal["*"], List[str], None] = None
    """Include all query string parameters."""


class ActionCacheKeyFieldsValueUser(BaseModel):
    """
    Feature fields to add features about the end-user (client) into
    the Cache Key.
    """

    device_type: Optional[bool] = None
    """
    Classifies a request as `mobile`, `desktop`, or `tablet` based on the User
    Agent.
    """

    geo: Optional[bool] = None
    """Includes the client's country, derived from the IP address."""

    lang: Optional[bool] = None
    """
    Includes the first language code contained in the `Accept-Language` header sent
    by the client.
    """


class ActionCacheKeyFieldsValue(BaseModel):
    cookie: Optional[ActionCacheKeyFieldsValueCookie] = None
    """Controls which cookies appear in the Cache Key."""

    header: Optional[ActionCacheKeyFieldsValueHeader] = None
    """Controls which headers go into the Cache Key.

    Exactly one of `include` or `exclude` is expected.
    """

    host: Optional[ActionCacheKeyFieldsValueHost] = None
    """Determines which host header to include in the Cache Key."""

    query_string: Optional[ActionCacheKeyFieldsValueQueryString] = None
    """Controls which URL query string parameters go into the Cache Key.

    Exactly one of `include` or `exclude` is expected.
    """

    user: Optional[ActionCacheKeyFieldsValueUser] = None
    """Feature fields to add features about the end-user (client) into the Cache Key."""


class ActionCacheKeyFields(BaseModel):
    id: Optional[Literal["cache_key_fields"]] = None
    """
    Control specifically what variables to include when deciding which resources to
    cache. This allows customers to determine what to cache based on something other
    than just the URL.
    """

    value: Optional[ActionCacheKeyFieldsValue] = None


class ActionCacheOnCookie(BaseModel):
    id: Optional[Literal["cache_on_cookie"]] = None
    """
    Apply the Cache Everything option (Cache Level setting) based on a regular
    expression match against a cookie name.
    """

    value: Optional[str] = None
    """The regular expression to use for matching cookie names in the request."""


class ActionCacheTTLByStatus(BaseModel):
    id: Optional[Literal["cache_ttl_by_status"]] = None
    """
    Enterprise customers can set cache time-to-live (TTL) based on the response
    status from the origin web server. Cache TTL refers to the duration of a
    resource in the Cloudflare network before being marked as stale or discarded
    from cache. Status codes are returned by a resource's origin. Setting cache TTL
    based on response status overrides the default cache behavior (standard caching)
    for static files and overrides cache instructions sent by the origin web server.
    To cache non-static assets, set a Cache Level of Cache Everything using a Page
    Rule. Setting no-store Cache-Control or a low TTL (using `max-age`/`s-maxage`)
    increases requests to origin web servers and decreases performance.
    """

    value: Optional[Dict[str, Union[Literal["no-cache", "no-store"], int]]] = None
    """
    A JSON object containing status codes and their corresponding TTLs. Each
    key-value pair in the cache TTL by status cache rule has the following syntax

    - `status_code`: An integer value such as 200 or 500. status_code matches the
      exact status code from the origin web server. Valid status codes are between
      100-999.
    - `status_code_range`: Integer values for from and to. status_code_range matches
      any status code from the origin web server within the specified range.
    - `value`: An integer value that defines the duration an asset is valid in
      seconds or one of the following strings: no-store (equivalent to -1), no-cache
      (equivalent to 0).
    """


class ActionDisableApps(BaseModel):
    id: Optional[Literal["disable_apps"]] = None
    """
    Turn off all active
    [Cloudflare Apps](https://developers.cloudflare.com/support/more-dashboard-apps/cloudflare-apps/)
    (deprecated).
    """


class ActionDisablePerformance(BaseModel):
    id: Optional[Literal["disable_performance"]] = None
    """
    Turn off
    [Rocket Loader](https://developers.cloudflare.com/speed/optimization/content/rocket-loader/),
    and [Polish](https://developers.cloudflare.com/images/polish/).
    """


class ActionDisableSecurity(BaseModel):
    id: Optional[Literal["disable_security"]] = None
    """
    Turn off
    [Email Obfuscation](https://developers.cloudflare.com/waf/tools/scrape-shield/email-address-obfuscation/),
    [Rate Limiting (previous version, deprecated)](https://developers.cloudflare.com/waf/reference/legacy/old-rate-limiting/),
    [Scrape Shield](https://developers.cloudflare.com/waf/tools/scrape-shield/),
    [URL (Zone) Lockdown](https://developers.cloudflare.com/waf/tools/zone-lockdown/),
    and
    [WAF managed rules (previous version, deprecated)](https://developers.cloudflare.com/waf/reference/legacy/old-waf-managed-rules/).
    """


class ActionDisableZaraz(BaseModel):
    id: Optional[Literal["disable_zaraz"]] = None
    """Turn off [Zaraz](https://developers.cloudflare.com/zaraz/)."""


class ActionEdgeCacheTTL(BaseModel):
    id: Optional[Literal["edge_cache_ttl"]] = None
    """Specify how long to cache a resource in the Cloudflare global network.

    _Edge Cache TTL_ is not visible in response headers.
    """

    value: Optional[int] = None


class ActionExplicitCacheControl(BaseModel):
    id: Optional[Literal["explicit_cache_control"]] = None
    """
    Origin Cache Control is enabled by default for Free, Pro, and Business domains
    and disabled by default for Enterprise domains.
    """

    value: Optional[Literal["on", "off"]] = None
    """The status of Origin Cache Control."""


class ActionForwardingURLValue(BaseModel):
    status_code: Optional[Literal[301, 302]] = None
    """The status code to use for the URL redirect.

    301 is a permanent redirect. 302 is a temporary redirect.
    """

    url: Optional[str] = None
    """
    The URL to redirect the request to. Notes: ${num} refers to the position of '\\**'
    in the constraint value.
    """


class ActionForwardingURL(BaseModel):
    id: Optional[Literal["forwarding_url"]] = None
    """Redirects one URL to another using an `HTTP 301/302` redirect.

    Refer to
    [Wildcard matching and referencing](https://developers.cloudflare.com/rules/page-rules/reference/wildcard-matching/).
    """

    value: Optional[ActionForwardingURLValue] = None


class ActionHostHeaderOverride(BaseModel):
    id: Optional[Literal["host_header_override"]] = None
    """Apply a specific host header."""

    value: Optional[str] = None
    """The hostname to use in the `Host` header"""


class ActionResolveOverride(BaseModel):
    id: Optional[Literal["resolve_override"]] = None
    """Change the origin address to the value specified in this setting."""

    value: Optional[str] = None
    """The origin address you want to override with."""


class ActionRespectStrongEtag(BaseModel):
    id: Optional[Literal["respect_strong_etag"]] = None
    """
    Turn on or off byte-for-byte equivalency checks between the Cloudflare cache and
    the origin server.
    """

    value: Optional[Literal["on", "off"]] = None
    """The status of Respect Strong ETags"""


Action: TypeAlias = Annotated[
    Union[
        AlwaysUseHTTPS,
        AutomaticHTTPSRewrites,
        BrowserCacheTTL,
        BrowserCheck,
        ActionBypassCacheOnCookie,
        ActionCacheByDeviceType,
        ActionCacheDeceptionArmor,
        ActionCacheKeyFields,
        CacheLevel,
        ActionCacheOnCookie,
        ActionCacheTTLByStatus,
        ActionDisableApps,
        ActionDisablePerformance,
        ActionDisableSecurity,
        ActionDisableZaraz,
        ActionEdgeCacheTTL,
        EmailObfuscation,
        ActionExplicitCacheControl,
        ActionForwardingURL,
        ActionHostHeaderOverride,
        IPGeolocation,
        Mirage,
        OpportunisticEncryption,
        OriginErrorPagePassThru,
        Polish,
        ActionResolveOverride,
        ActionRespectStrongEtag,
        ResponseBuffering,
        RocketLoader,
        SecurityLevel,
        SortQueryStringForCache,
        SSL,
        TrueClientIPHeader,
        WAF,
    ],
    PropertyInfo(discriminator="id"),
]


class PageRule(BaseModel):
    id: str
    """Identifier."""

    actions: List[Action]
    """The set of actions to perform if the targets of this rule match the request.

    Actions can redirect to another URL or override settings, but not both.
    """

    created_on: datetime
    """The timestamp of when the Page Rule was created."""

    modified_on: datetime
    """The timestamp of when the Page Rule was last modified."""

    priority: int
    """
    The priority of the rule, used to define which Page Rule is processed over
    another. A higher number indicates a higher priority. For example, if you have a
    catch-all Page Rule (rule A: `/images/*`) but want a more specific Page Rule to
    take precedence (rule B: `/images/special/*`), specify a higher priority for
    rule B so it overrides rule A.
    """

    status: Literal["active", "disabled"]
    """The status of the Page Rule."""

    targets: List[Target]
    """The rule targets to evaluate on each request."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "DynamicRoutingDeleteResponse",
    "Element",
    "ElementUnionMember0",
    "ElementUnionMember0Outputs",
    "ElementUnionMember0OutputsNext",
    "ElementUnionMember1",
    "ElementUnionMember1Outputs",
    "ElementUnionMember1OutputsFalse_",
    "ElementUnionMember1OutputsTrue_",
    "ElementUnionMember1Properties",
    "ElementUnionMember2",
    "ElementUnionMember2Outputs",
    "ElementUnionMember3",
    "ElementUnionMember3Outputs",
    "ElementUnionMember3OutputsFallback",
    "ElementUnionMember3OutputsSuccess",
    "ElementUnionMember3Properties",
    "ElementUnionMember4",
    "ElementUnionMember4Outputs",
    "ElementUnionMember4OutputsFallback",
    "ElementUnionMember4OutputsSuccess",
    "ElementUnionMember4Properties",
    "ElementUnionMember5",
    "ElementUnionMember5Outputs",
]


class ElementUnionMember0OutputsNext(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class ElementUnionMember0Outputs(BaseModel):
    next: ElementUnionMember0OutputsNext


class ElementUnionMember0(BaseModel):
    id: str

    outputs: ElementUnionMember0Outputs

    type: Literal["start"]


class ElementUnionMember1OutputsFalse_(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class ElementUnionMember1OutputsTrue_(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class ElementUnionMember1Outputs(BaseModel):
    false: ElementUnionMember1OutputsFalse_

    true: ElementUnionMember1OutputsTrue_


class ElementUnionMember1Properties(BaseModel):
    conditions: Optional[object] = None


class ElementUnionMember1(BaseModel):
    id: str

    outputs: ElementUnionMember1Outputs

    properties: ElementUnionMember1Properties

    type: Literal["conditional"]


class ElementUnionMember2Outputs(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class ElementUnionMember2(BaseModel):
    id: str

    outputs: Dict[str, ElementUnionMember2Outputs]

    type: Literal["percentage"]


class ElementUnionMember3OutputsFallback(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class ElementUnionMember3OutputsSuccess(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class ElementUnionMember3Outputs(BaseModel):
    fallback: ElementUnionMember3OutputsFallback

    success: ElementUnionMember3OutputsSuccess


class ElementUnionMember3Properties(BaseModel):
    key: str

    limit: float

    limit_type: Literal["count", "cost"] = FieldInfo(alias="limitType")

    window: float


class ElementUnionMember3(BaseModel):
    id: str

    outputs: ElementUnionMember3Outputs

    properties: ElementUnionMember3Properties

    type: Literal["rate"]


class ElementUnionMember4OutputsFallback(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class ElementUnionMember4OutputsSuccess(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class ElementUnionMember4Outputs(BaseModel):
    fallback: ElementUnionMember4OutputsFallback

    success: ElementUnionMember4OutputsSuccess


class ElementUnionMember4Properties(BaseModel):
    model: str

    provider: str

    retries: float

    timeout: float


class ElementUnionMember4(BaseModel):
    id: str

    outputs: ElementUnionMember4Outputs

    properties: ElementUnionMember4Properties

    type: Literal["model"]


class ElementUnionMember5Outputs(BaseModel):
    element_id: str = FieldInfo(alias="elementId")


class ElementUnionMember5(BaseModel):
    id: str

    outputs: Dict[str, ElementUnionMember5Outputs]

    type: Literal["end"]


Element: TypeAlias = Union[
    ElementUnionMember0,
    ElementUnionMember1,
    ElementUnionMember2,
    ElementUnionMember3,
    ElementUnionMember4,
    ElementUnionMember5,
]


class DynamicRoutingDeleteResponse(BaseModel):
    id: str

    created_at: datetime

    elements: List[Element]

    gateway_id: str

    modified_at: datetime

    name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import TypeAlias

from ...._models import BaseModel

__all__ = ["AppType", "ZeroTrustGatewayApplication", "ZeroTrustGatewayApplicationType"]


class ZeroTrustGatewayApplication(BaseModel):
    id: Optional[int] = None
    """Identify this application. Only one application per ID."""

    application_type_id: Optional[int] = None
    """Identify the type of this application.

    Multiple applications can share the same type. Refers to the `id` of a returned
    application type.
    """

    created_at: Optional[datetime] = None

    name: Optional[str] = None
    """Specify the name of the application or application type."""


class ZeroTrustGatewayApplicationType(BaseModel):
    id: Optional[int] = None
    """Identify the type of this application.

    Multiple applications can share the same type. Refers to the `id` of a returned
    application type.
    """

    created_at: Optional[datetime] = None

    description: Optional[str] = None
    """Provide a short summary of applications with this type."""

    name: Optional[str] = None
    """Specify the name of the application or application type."""


AppType: TypeAlias = Union[ZeroTrustGatewayApplication, ZeroTrustGatewayApplicationType]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["SiteLocation"]


class SiteLocation(BaseModel):
    """Location of site in latitude and longitude."""

    lat: Optional[str] = None
    """Latitude"""

    lon: Optional[str] = None
    """Longitude"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .objects import (
    ObjectsResource,
    AsyncObjectsResource,
    ObjectsResourceWithRawResponse,
    AsyncObjectsResourceWithRawResponse,
    ObjectsResourceWithStreamingResponse,
    AsyncObjectsResourceWithStreamingResponse,
)
from .namespaces import (
    NamespacesResource,
    AsyncNamespacesResource,
    NamespacesResourceWithRawResponse,
    AsyncNamespacesResourceWithRawResponse,
    NamespacesResourceWithStreamingResponse,
    AsyncNamespacesResourceWithStreamingResponse,
)

__all__ = [
    "ObjectsResource",
    "AsyncObjectsResource",
    "ObjectsResourceWithRawResponse",
    "AsyncObjectsResourceWithRawResponse",
    "ObjectsResourceWithStreamingResponse",
    "AsyncObjectsResourceWithStreamingResponse",
    "NamespacesResource",
    "AsyncNamespacesResource",
    "NamespacesResourceWithRawResponse",
    "AsyncNamespacesResourceWithRawResponse",
    "NamespacesResourceWithStreamingResponse",
    "AsyncNamespacesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["BrowserVersionResponse"]


class BrowserVersionResponse(BaseModel):
    browser: str = FieldInfo(alias="Browser")
    """Browser name and version."""

    protocol_version: str = FieldInfo(alias="Protocol-Version")
    """Chrome DevTools Protocol version."""

    user_agent: str = FieldInfo(alias="User-Agent")
    """User agent string."""

    v8_version: str = FieldInfo(alias="V8-Version")
    """V8 JavaScript engine version."""

    web_kit_version: str = FieldInfo(alias="WebKit-Version")
    """WebKit version."""

    web_socket_debugger_url: str = FieldInfo(alias="webSocketDebuggerUrl")
    """WebSocket URL for debugging the browser."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["MeetingRefreshParticipantTokenResponse", "Data"]


class Data(BaseModel):
    """Data returned by the operation"""

    token: str
    """Regenerated participant's authentication token."""


class MeetingRefreshParticipantTokenResponse(BaseModel):
    data: Data
    """Data returned by the operation"""

    success: bool
    """Success status of the operation"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["UserEditParams"]


class UserEditParams(TypedDict, total=False):
    country: Optional[str]
    """The country in which the user lives."""

    first_name: Optional[str]
    """User's first name"""

    last_name: Optional[str]
    """User's last name"""

    telephone: Optional[str]
    """User's telephone number"""

    zipcode: Optional[str]
    """The zipcode or postal code where the user lives."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["Logging"]


class Logging(BaseModel):
    """An object configuring the rule's logging behavior."""

    enabled: bool
    """Whether to generate a log when the rule matches."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast

import httpx

from .config import (
    ConfigResource,
    AsyncConfigResource,
    ConfigResourceWithRawResponse,
    AsyncConfigResourceWithRawResponse,
    ConfigResourceWithStreamingResponse,
    AsyncConfigResourceWithStreamingResponse,
)
from .export import (
    ExportResource,
    AsyncExportResource,
    ExportResourceWithRawResponse,
    AsyncExportResourceWithRawResponse,
    ExportResourceWithStreamingResponse,
    AsyncExportResourceWithStreamingResponse,
)
from .default import (
    DefaultResource,
    AsyncDefaultResource,
    DefaultResourceWithRawResponse,
    AsyncDefaultResourceWithRawResponse,
    DefaultResourceWithStreamingResponse,
    AsyncDefaultResourceWithStreamingResponse,
)
from .publish import (
    PublishResource,
    AsyncPublishResource,
    PublishResourceWithRawResponse,
    AsyncPublishResourceWithRawResponse,
    PublishResourceWithStreamingResponse,
    AsyncPublishResourceWithStreamingResponse,
)
from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from .workflow import (
    WorkflowResource,
    AsyncWorkflowResource,
    WorkflowResourceWithRawResponse,
    AsyncWorkflowResourceWithRawResponse,
    WorkflowResourceWithStreamingResponse,
    AsyncWorkflowResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.zaraz import Workflow, zaraz_update_params
from ..._base_client import make_request_options
from .history.history import (
    HistoryResource,
    AsyncHistoryResource,
    HistoryResourceWithRawResponse,
    AsyncHistoryResourceWithRawResponse,
    HistoryResourceWithStreamingResponse,
    AsyncHistoryResourceWithStreamingResponse,
)
from ...types.zaraz.workflow import Workflow

__all__ = ["ZarazResource", "AsyncZarazResource"]


class ZarazResource(SyncAPIResource):
    @cached_property
    def config(self) -> ConfigResource:
        return ConfigResource(self._client)

    @cached_property
    def default(self) -> DefaultResource:
        return DefaultResource(self._client)

    @cached_property
    def export(self) -> ExportResource:
        return ExportResource(self._client)

    @cached_property
    def history(self) -> HistoryResource:
        return HistoryResource(self._client)

    @cached_property
    def publish(self) -> PublishResource:
        return PublishResource(self._client)

    @cached_property
    def workflow(self) -> WorkflowResource:
        return WorkflowResource(self._client)

    @cached_property
    def with_raw_response(self) -> ZarazResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ZarazResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ZarazResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ZarazResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        zone_id: str,
        workflow: Workflow,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Workflow:
        """
        Updates Zaraz workflow for a zone.

        Args:
          zone_id: Identifier.

          workflow: Zaraz workflow.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/settings/zaraz/workflow", zone_id=zone_id),
            body=maybe_transform(workflow, zaraz_update_params.ZarazUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Workflow]._unwrapper,
            ),
            cast_to=cast(
                Any, ResultWrapper[Workflow]
            ),  # Enum types cannot be passed in as arguments in the type system
        )


class AsyncZarazResource(AsyncAPIResource):
    @cached_property
    def config(self) -> AsyncConfigResource:
        return AsyncConfigResource(self._client)

    @cached_property
    def default(self) -> AsyncDefaultResource:
        return AsyncDefaultResource(self._client)

    @cached_property
    def export(self) -> AsyncExportResource:
        return AsyncExportResource(self._client)

    @cached_property
    def history(self) -> AsyncHistoryResource:
        return AsyncHistoryResource(self._client)

    @cached_property
    def publish(self) -> AsyncPublishResource:
        return AsyncPublishResource(self._client)

    @cached_property
    def workflow(self) -> AsyncWorkflowResource:
        return AsyncWorkflowResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncZarazResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncZarazResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncZarazResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncZarazResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        zone_id: str,
        workflow: Workflow,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Workflow:
        """
        Updates Zaraz workflow for a zone.

        Args:
          zone_id: Identifier.

          workflow: Zaraz workflow.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/settings/zaraz/workflow", zone_id=zone_id),
            body=await async_maybe_transform(workflow, zaraz_update_params.ZarazUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Workflow]._unwrapper,
            ),
            cast_to=cast(
                Any, ResultWrapper[Workflow]
            ),  # Enum types cannot be passed in as arguments in the type system
        )


class ZarazResourceWithRawResponse:
    def __init__(self, zaraz: ZarazResource) -> None:
        self._zaraz = zaraz

        self.update = to_raw_response_wrapper(
            zaraz.update,
        )

    @cached_property
    def config(self) -> ConfigResourceWithRawResponse:
        return ConfigResourceWithRawResponse(self._zaraz.config)

    @cached_property
    def default(self) -> DefaultResourceWithRawResponse:
        return DefaultResourceWithRawResponse(self._zaraz.default)

    @cached_property
    def export(self) -> ExportResourceWithRawResponse:
        return ExportResourceWithRawResponse(self._zaraz.export)

    @cached_property
    def history(self) -> HistoryResourceWithRawResponse:
        return HistoryResourceWithRawResponse(self._zaraz.history)

    @cached_property
    def publish(self) -> PublishResourceWithRawResponse:
        return PublishResourceWithRawResponse(self._zaraz.publish)

    @cached_property
    def workflow(self) -> WorkflowResourceWithRawResponse:
        return WorkflowResourceWithRawResponse(self._zaraz.workflow)


class AsyncZarazResourceWithRawResponse:
    def __init__(self, zaraz: AsyncZarazResource) -> None:
        self._zaraz = zaraz

        self.update = async_to_raw_response_wrapper(
            zaraz.update,
        )

    @cached_property
    def config(self) -> AsyncConfigResourceWithRawResponse:
        return AsyncConfigResourceWithRawResponse(self._zaraz.config)

    @cached_property
    def default(self) -> AsyncDefaultResourceWithRawResponse:
        return AsyncDefaultResourceWithRawResponse(self._zaraz.default)

    @cached_property
    def export(self) -> AsyncExportResourceWithRawResponse:
        return AsyncExportResourceWithRawResponse(self._zaraz.export)

    @cached_property
    def history(self) -> AsyncHistoryResourceWithRawResponse:
        return AsyncHistoryResourceWithRawResponse(self._zaraz.history)

    @cached_property
    def publish(self) -> AsyncPublishResourceWithRawResponse:
        return AsyncPublishResourceWithRawResponse(self._zaraz.publish)

    @cached_property
    def workflow(self) -> AsyncWorkflowResourceWithRawResponse:
        return AsyncWorkflowResourceWithRawResponse(self._zaraz.workflow)


class ZarazResourceWithStreamingResponse:
    def __init__(self, zaraz: ZarazResource) -> None:
        self._zaraz = zaraz

        self.update = to_streamed_response_wrapper(
            zaraz.update,
        )

    @cached_property
    def config(self) -> ConfigResourceWithStreamingResponse:
        return ConfigResourceWithStreamingResponse(self._zaraz.config)

    @cached_property
    def default(self) -> DefaultResourceWithStreamingResponse:
        return DefaultResourceWithStreamingResponse(self._zaraz.default)

    @cached_property
    def export(self) -> ExportResourceWithStreamingResponse:
        return ExportResourceWithStreamingResponse(self._zaraz.export)

    @cached_property
    def history(self) -> HistoryResourceWithStreamingResponse:
        return HistoryResourceWithStreamingResponse(self._zaraz.history)

    @cached_property
    def publish(self) -> PublishResourceWithStreamingResponse:
        return PublishResourceWithStreamingResponse(self._zaraz.publish)

    @cached_property
    def workflow(self) -> WorkflowResourceWithStreamingResponse:
        return WorkflowResourceWithStreamingResponse(self._zaraz.workflow)


class AsyncZarazResourceWithStreamingResponse:
    def __init__(self, zaraz: AsyncZarazResource) -> None:
        self._zaraz = zaraz

        self.update = async_to_streamed_response_wrapper(
            zaraz.update,
        )

    @cached_property
    def config(self) -> AsyncConfigResourceWithStreamingResponse:
        return AsyncConfigResourceWithStreamingResponse(self._zaraz.config)

    @cached_property
    def default(self) -> AsyncDefaultResourceWithStreamingResponse:
        return AsyncDefaultResourceWithStreamingResponse(self._zaraz.default)

    @cached_property
    def export(self) -> AsyncExportResourceWithStreamingResponse:
        return AsyncExportResourceWithStreamingResponse(self._zaraz.export)

    @cached_property
    def history(self) -> AsyncHistoryResourceWithStreamingResponse:
        return AsyncHistoryResourceWithStreamingResponse(self._zaraz.history)

    @cached_property
    def publish(self) -> AsyncPublishResourceWithStreamingResponse:
        return AsyncPublishResourceWithStreamingResponse(self._zaraz.publish)

    @cached_property
    def workflow(self) -> AsyncWorkflowResourceWithStreamingResponse:
        return AsyncWorkflowResourceWithStreamingResponse(self._zaraz.workflow)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.connectivity.directory import service_list_params, service_create_params, service_update_params
from ....types.connectivity.directory.service_get_response import ServiceGetResponse
from ....types.connectivity.directory.service_list_response import ServiceListResponse
from ....types.connectivity.directory.service_create_response import ServiceCreateResponse
from ....types.connectivity.directory.service_update_response import ServiceUpdateResponse

__all__ = ["ServicesResource", "AsyncServicesResource"]


class ServicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ServicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ServicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ServicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ServicesResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        account_id: str,
        host: service_create_params.InfraHTTPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        http_port: Optional[int] | Omit = omit,
        https_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_create_params.InfraHTTPServiceConfigTLSSettings] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceCreateResponse]:
        """
        Create Workers VPC connectivity service

        Args:
          account_id: Account identifier

          tls_settings: TLS settings for a connectivity service.

              If omitted, the default mode (`verify_full`) is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        host: service_create_params.InfraTCPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        app_protocol: Optional[Literal["postgresql", "mysql"]] | Omit = omit,
        tcp_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_create_params.InfraTCPServiceConfigTLSSettings] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceCreateResponse]:
        """
        Create Workers VPC connectivity service

        Args:
          account_id: Account identifier

          tls_settings: TLS settings for a connectivity service.

              If omitted, the default mode (`verify_full`) is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "host", "name", "type"])
    def create(
        self,
        *,
        account_id: str,
        host: service_create_params.InfraHTTPServiceConfigHost | service_create_params.InfraTCPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        http_port: Optional[int] | Omit = omit,
        https_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_create_params.InfraHTTPServiceConfigTLSSettings]
        | Optional[service_create_params.InfraTCPServiceConfigTLSSettings]
        | Omit = omit,
        app_protocol: Optional[Literal["postgresql", "mysql"]] | Omit = omit,
        tcp_port: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceCreateResponse]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            Optional[ServiceCreateResponse],
            self._post(
                path_template("/accounts/{account_id}/connectivity/directory/services", account_id=account_id),
                body=maybe_transform(
                    {
                        "host": host,
                        "name": name,
                        "type": type,
                        "http_port": http_port,
                        "https_port": https_port,
                        "tls_settings": tls_settings,
                        "app_protocol": app_protocol,
                        "tcp_port": tcp_port,
                    },
                    service_create_params.ServiceCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ServiceCreateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ServiceCreateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    def update(
        self,
        service_id: str,
        *,
        account_id: str,
        host: service_update_params.InfraHTTPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        http_port: Optional[int] | Omit = omit,
        https_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_update_params.InfraHTTPServiceConfigTLSSettings] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceUpdateResponse]:
        """
        Update Workers VPC connectivity service

        Args:
          tls_settings: TLS settings for a connectivity service.

              If omitted, the default mode (`verify_full`) is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        service_id: str,
        *,
        account_id: str,
        host: service_update_params.InfraTCPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        app_protocol: Optional[Literal["postgresql", "mysql"]] | Omit = omit,
        tcp_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_update_params.InfraTCPServiceConfigTLSSettings] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceUpdateResponse]:
        """
        Update Workers VPC connectivity service

        Args:
          tls_settings: TLS settings for a connectivity service.

              If omitted, the default mode (`verify_full`) is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "host", "name", "type"])
    def update(
        self,
        service_id: str,
        *,
        account_id: str,
        host: service_update_params.InfraHTTPServiceConfigHost | service_update_params.InfraTCPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        http_port: Optional[int] | Omit = omit,
        https_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_update_params.InfraHTTPServiceConfigTLSSettings]
        | Optional[service_update_params.InfraTCPServiceConfigTLSSettings]
        | Omit = omit,
        app_protocol: Optional[Literal["postgresql", "mysql"]] | Omit = omit,
        tcp_port: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceUpdateResponse]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not service_id:
            raise ValueError(f"Expected a non-empty value for `service_id` but received {service_id!r}")
        return cast(
            Optional[ServiceUpdateResponse],
            self._put(
                path_template(
                    "/accounts/{account_id}/connectivity/directory/services/{service_id}",
                    account_id=account_id,
                    service_id=service_id,
                ),
                body=maybe_transform(
                    {
                        "host": host,
                        "name": name,
                        "type": type,
                        "http_port": http_port,
                        "https_port": https_port,
                        "tls_settings": tls_settings,
                        "app_protocol": app_protocol,
                        "tcp_port": tcp_port,
                    },
                    service_update_params.ServiceUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ServiceUpdateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ServiceUpdateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        type: Optional[Literal["tcp", "http"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[ServiceListResponse]:
        """
        List Workers VPC connectivity services

        Args:
          account_id: Account identifier

          page: Current page in the response

          per_page: Max amount of entries returned per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/connectivity/directory/services", account_id=account_id),
            page=SyncV4PagePaginationArray[ServiceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "type": type,
                    },
                    service_list_params.ServiceListParams,
                ),
            ),
            model=cast(Any, ServiceListResponse),  # Union types cannot be passed in as arguments in the type system
        )

    def delete(
        self,
        service_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete Workers VPC connectivity service

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not service_id:
            raise ValueError(f"Expected a non-empty value for `service_id` but received {service_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template(
                "/accounts/{account_id}/connectivity/directory/services/{service_id}",
                account_id=account_id,
                service_id=service_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        service_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceGetResponse]:
        """
        Get Workers VPC connectivity service

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not service_id:
            raise ValueError(f"Expected a non-empty value for `service_id` but received {service_id!r}")
        return cast(
            Optional[ServiceGetResponse],
            self._get(
                path_template(
                    "/accounts/{account_id}/connectivity/directory/services/{service_id}",
                    account_id=account_id,
                    service_id=service_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ServiceGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ServiceGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncServicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncServicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncServicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncServicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncServicesResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        account_id: str,
        host: service_create_params.InfraHTTPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        http_port: Optional[int] | Omit = omit,
        https_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_create_params.InfraHTTPServiceConfigTLSSettings] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceCreateResponse]:
        """
        Create Workers VPC connectivity service

        Args:
          account_id: Account identifier

          tls_settings: TLS settings for a connectivity service.

              If omitted, the default mode (`verify_full`) is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        host: service_create_params.InfraTCPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        app_protocol: Optional[Literal["postgresql", "mysql"]] | Omit = omit,
        tcp_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_create_params.InfraTCPServiceConfigTLSSettings] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceCreateResponse]:
        """
        Create Workers VPC connectivity service

        Args:
          account_id: Account identifier

          tls_settings: TLS settings for a connectivity service.

              If omitted, the default mode (`verify_full`) is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "host", "name", "type"])
    async def create(
        self,
        *,
        account_id: str,
        host: service_create_params.InfraHTTPServiceConfigHost | service_create_params.InfraTCPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        http_port: Optional[int] | Omit = omit,
        https_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_create_params.InfraHTTPServiceConfigTLSSettings]
        | Optional[service_create_params.InfraTCPServiceConfigTLSSettings]
        | Omit = omit,
        app_protocol: Optional[Literal["postgresql", "mysql"]] | Omit = omit,
        tcp_port: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceCreateResponse]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            Optional[ServiceCreateResponse],
            await self._post(
                path_template("/accounts/{account_id}/connectivity/directory/services", account_id=account_id),
                body=await async_maybe_transform(
                    {
                        "host": host,
                        "name": name,
                        "type": type,
                        "http_port": http_port,
                        "https_port": https_port,
                        "tls_settings": tls_settings,
                        "app_protocol": app_protocol,
                        "tcp_port": tcp_port,
                    },
                    service_create_params.ServiceCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ServiceCreateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ServiceCreateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    async def update(
        self,
        service_id: str,
        *,
        account_id: str,
        host: service_update_params.InfraHTTPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        http_port: Optional[int] | Omit = omit,
        https_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_update_params.InfraHTTPServiceConfigTLSSettings] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceUpdateResponse]:
        """
        Update Workers VPC connectivity service

        Args:
          tls_settings: TLS settings for a connectivity service.

              If omitted, the default mode (`verify_full`) is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        service_id: str,
        *,
        account_id: str,
        host: service_update_params.InfraTCPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        app_protocol: Optional[Literal["postgresql", "mysql"]] | Omit = omit,
        tcp_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_update_params.InfraTCPServiceConfigTLSSettings] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceUpdateResponse]:
        """
        Update Workers VPC connectivity service

        Args:
          tls_settings: TLS settings for a connectivity service.

              If omitted, the default mode (`verify_full`) is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "host", "name", "type"])
    async def update(
        self,
        service_id: str,
        *,
        account_id: str,
        host: service_update_params.InfraHTTPServiceConfigHost | service_update_params.InfraTCPServiceConfigHost,
        name: str,
        type: Literal["tcp", "http"],
        http_port: Optional[int] | Omit = omit,
        https_port: Optional[int] | Omit = omit,
        tls_settings: Optional[service_update_params.InfraHTTPServiceConfigTLSSettings]
        | Optional[service_update_params.InfraTCPServiceConfigTLSSettings]
        | Omit = omit,
        app_protocol: Optional[Literal["postgresql", "mysql"]] | Omit = omit,
        tcp_port: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceUpdateResponse]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not service_id:
            raise ValueError(f"Expected a non-empty value for `service_id` but received {service_id!r}")
        return cast(
            Optional[ServiceUpdateResponse],
            await self._put(
                path_template(
                    "/accounts/{account_id}/connectivity/directory/services/{service_id}",
                    account_id=account_id,
                    service_id=service_id,
                ),
                body=await async_maybe_transform(
                    {
                        "host": host,
                        "name": name,
                        "type": type,
                        "http_port": http_port,
                        "https_port": https_port,
                        "tls_settings": tls_settings,
                        "app_protocol": app_protocol,
                        "tcp_port": tcp_port,
                    },
                    service_update_params.ServiceUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ServiceUpdateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ServiceUpdateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        type: Optional[Literal["tcp", "http"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ServiceListResponse, AsyncV4PagePaginationArray[ServiceListResponse]]:
        """
        List Workers VPC connectivity services

        Args:
          account_id: Account identifier

          page: Current page in the response

          per_page: Max amount of entries returned per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/connectivity/directory/services", account_id=account_id),
            page=AsyncV4PagePaginationArray[ServiceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "type": type,
                    },
                    service_list_params.ServiceListParams,
                ),
            ),
            model=cast(Any, ServiceListResponse),  # Union types cannot be passed in as arguments in the type system
        )

    async def delete(
        self,
        service_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete Workers VPC connectivity service

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not service_id:
            raise ValueError(f"Expected a non-empty value for `service_id` but received {service_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template(
                "/accounts/{account_id}/connectivity/directory/services/{service_id}",
                account_id=account_id,
                service_id=service_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        service_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceGetResponse]:
        """
        Get Workers VPC connectivity service

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not service_id:
            raise ValueError(f"Expected a non-empty value for `service_id` but received {service_id!r}")
        return cast(
            Optional[ServiceGetResponse],
            await self._get(
                path_template(
                    "/accounts/{account_id}/connectivity/directory/services/{service_id}",
                    account_id=account_id,
                    service_id=service_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ServiceGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ServiceGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class ServicesResourceWithRawResponse:
    def __init__(self, services: ServicesResource) -> None:
        self._services = services

        self.create = to_raw_response_wrapper(
            services.create,
        )
        self.update = to_raw_response_wrapper(
            services.update,
        )
        self.list = to_raw_response_wrapper(
            services.list,
        )
        self.delete = to_raw_response_wrapper(
            services.delete,
        )
        self.get = to_raw_response_wrapper(
            services.get,
        )


class AsyncServicesResourceWithRawResponse:
    def __init__(self, services: AsyncServicesResource) -> None:
        self._services = services

        self.create = async_to_raw_response_wrapper(
            services.create,
        )
        self.update = async_to_raw_response_wrapper(
            services.update,
        )
        self.list = async_to_raw_response_wrapper(
            services.list,
        )
        self.delete = async_to_raw_response_wrapper(
            services.delete,
        )
        self.get = async_to_raw_response_wrapper(
            services.get,
        )


class ServicesResourceWithStreamingResponse:
    def __init__(self, services: ServicesResource) -> None:
        self._services = services

        self.create = to_streamed_response_wrapper(
            services.create,
        )
        self.update = to_streamed_response_wrapper(
            services.update,
        )
        self.list = to_streamed_response_wrapper(
            services.list,
        )
        self.delete = to_streamed_response_wrapper(
            services.delete,
        )
        self.get = to_streamed_response_wrapper(
            services.get,
        )


class AsyncServicesResourceWithStreamingResponse:
    def __init__(self, services: AsyncServicesResource) -> None:
        self._services = services

        self.create = async_to_streamed_response_wrapper(
            services.create,
        )
        self.update = async_to_streamed_response_wrapper(
            services.update,
        )
        self.list = async_to_streamed_response_wrapper(
            services.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            services.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            services.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import datetime
from typing import Union
from typing_extensions import Literal, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["AgentReadinessSummaryParams"]


class AgentReadinessSummaryParams(TypedDict, total=False):
    date: Annotated[Union[str, datetime.date], PropertyInfo(format="iso8601")]
    """Filters results by the specified date."""

    domain_category: Annotated[SequenceNotStr[str], PropertyInfo(alias="domainCategory")]
    """Filters results by domain category."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    name: SequenceNotStr[str]
    """Array of names used to label the series in the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ......_types import Body, Query, Headers, NotGiven, not_given
from ......_utils import path_template
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......_base_client import make_request_options
from ......types.pages.projects.deployments.history.log_get_response import LogGetResponse

__all__ = ["LogsResource", "AsyncLogsResource"]


class LogsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LogsResourceWithStreamingResponse(self)

    def get(
        self,
        deployment_id: str,
        *,
        account_id: str,
        project_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogGetResponse:
        """
        Fetch deployment logs for a project.

        Args:
          account_id: Identifier.

          project_name: Name of the project.

          deployment_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not project_name:
            raise ValueError(f"Expected a non-empty value for `project_name` but received {project_name!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/pages/projects/{project_name}/deployments/{deployment_id}/history/logs",
                account_id=account_id,
                project_name=project_name,
                deployment_id=deployment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LogGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[LogGetResponse], ResultWrapper[LogGetResponse]),
        )


class AsyncLogsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLogsResourceWithStreamingResponse(self)

    async def get(
        self,
        deployment_id: str,
        *,
        account_id: str,
        project_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogGetResponse:
        """
        Fetch deployment logs for a project.

        Args:
          account_id: Identifier.

          project_name: Name of the project.

          deployment_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not project_name:
            raise ValueError(f"Expected a non-empty value for `project_name` but received {project_name!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/pages/projects/{project_name}/deployments/{deployment_id}/history/logs",
                account_id=account_id,
                project_name=project_name,
                deployment_id=deployment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LogGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[LogGetResponse], ResultWrapper[LogGetResponse]),
        )


class LogsResourceWithRawResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

        self.get = to_raw_response_wrapper(
            logs.get,
        )


class AsyncLogsResourceWithRawResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

        self.get = async_to_raw_response_wrapper(
            logs.get,
        )


class LogsResourceWithStreamingResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

        self.get = to_streamed_response_wrapper(
            logs.get,
        )


class AsyncLogsResourceWithStreamingResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

        self.get = async_to_streamed_response_wrapper(
            logs.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ..gateway_item import GatewayItem

__all__ = ["ItemListResponse"]

ItemListResponse: TypeAlias = List[GatewayItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["PreviewCreateResponse"]


class PreviewCreateResponse(BaseModel):
    screenshot: str
    """A base64 encoded PNG image of the email."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._utils import PropertyInfo

__all__ = [
    "LockUpdateParams",
    "Rule",
    "RuleCondition",
    "RuleConditionR2LockRuleAgeCondition",
    "RuleConditionR2LockRuleDateCondition",
    "RuleConditionR2LockRuleIndefiniteCondition",
]


class LockUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    rules: Iterable[Rule]

    jurisdiction: Annotated[Literal["default", "eu", "fedramp"], PropertyInfo(alias="cf-r2-jurisdiction")]
    """Jurisdiction where objects in this bucket are guaranteed to be stored."""


class RuleConditionR2LockRuleAgeCondition(TypedDict, total=False):
    """Condition to apply a lock rule to an object for how long in seconds."""

    max_age_seconds: Required[Annotated[int, PropertyInfo(alias="maxAgeSeconds")]]

    type: Required[Literal["Age"]]


class RuleConditionR2LockRuleDateCondition(TypedDict, total=False):
    """Condition to apply a lock rule to an object until a specific date."""

    date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    type: Required[Literal["Date"]]


class RuleConditionR2LockRuleIndefiniteCondition(TypedDict, total=False):
    """Condition to apply a lock rule indefinitely."""

    type: Required[Literal["Indefinite"]]


RuleCondition: TypeAlias = Union[
    RuleConditionR2LockRuleAgeCondition,
    RuleConditionR2LockRuleDateCondition,
    RuleConditionR2LockRuleIndefiniteCondition,
]


class Rule(TypedDict, total=False):
    id: Required[str]
    """Unique identifier for this rule."""

    condition: Required[RuleCondition]
    """Condition to apply a lock rule to an object for how long in seconds."""

    enabled: Required[bool]
    """Whether or not this rule is in effect."""

    prefix: str
    """
    Rule will only apply to objects/uploads in the bucket that start with the given
    prefix, an empty prefix can be provided to scope rule to all objects/uploads.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["HistoryListResponse"]


class HistoryListResponse(BaseModel):
    id: int
    """ID of the configuration."""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Date and time the configuration was created."""

    description: str
    """
    Configuration description provided by the user who published this configuration.
    """

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Date and time the configuration was last updated."""

    user_id: str = FieldInfo(alias="userId")
    """Alpha-numeric ID of the account user who published the configuration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["DatasetCreateParams", "Filter"]


class DatasetCreateParams(TypedDict, total=False):
    account_id: Required[str]

    enable: Required[bool]

    filters: Required[Iterable[Filter]]

    name: Required[str]


class Filter(TypedDict, total=False):
    key: Required[
        Literal[
            "created_at",
            "request_content_type",
            "response_content_type",
            "success",
            "cached",
            "provider",
            "model",
            "cost",
            "tokens",
            "tokens_in",
            "tokens_out",
            "duration",
            "feedback",
        ]
    ]

    operator: Required[Literal["eq", "contains", "lt", "gt"]]

    value: Required[SequenceNotStr[Union[str, float, bool]]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from .ttl_param import TTLParam
from .record_tags import RecordTags

__all__ = [
    "RecordUpdateParams",
    "ARecord",
    "ARecordSettings",
    "AAAARecord",
    "AAAARecordSettings",
    "CNAMERecord",
    "CNAMERecordSettings",
    "MXRecord",
    "MXRecordSettings",
    "NSRecord",
    "NSRecordSettings",
    "DNSRecordsOpenpgpkeyRecord",
    "DNSRecordsOpenpgpkeyRecordSettings",
    "PTRRecord",
    "PTRRecordSettings",
    "TXTRecord",
    "TXTRecordSettings",
    "CAARecord",
    "CAARecordData",
    "CAARecordSettings",
    "CERTRecord",
    "CERTRecordData",
    "CERTRecordSettings",
    "DNSKEYRecord",
    "DNSKEYRecordData",
    "DNSKEYRecordSettings",
    "DSRecord",
    "DSRecordData",
    "DSRecordSettings",
    "HTTPSRecord",
    "HTTPSRecordData",
    "HTTPSRecordSettings",
    "LOCRecord",
    "LOCRecordData",
    "LOCRecordSettings",
    "NAPTRRecord",
    "NAPTRRecordData",
    "NAPTRRecordSettings",
    "SMIMEARecord",
    "SMIMEARecordData",
    "SMIMEARecordSettings",
    "SRVRecord",
    "SRVRecordData",
    "SRVRecordSettings",
    "SSHFPRecord",
    "SSHFPRecordData",
    "SSHFPRecordSettings",
    "SVCBRecord",
    "SVCBRecordData",
    "SVCBRecordSettings",
    "TLSARecord",
    "TLSARecordData",
    "TLSARecordSettings",
    "URIRecord",
    "URIRecordData",
    "URIRecordSettings",
]


class ARecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["A"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """A valid IPv4 address."""

    private_routing: bool
    """Enables private network routing to the origin."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: ARecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class ARecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class AAAARecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["AAAA"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """A valid IPv6 address."""

    private_routing: bool
    """Enables private network routing to the origin."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: AAAARecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class AAAARecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class CNAMERecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["CNAME"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """A valid hostname. Must not match the record's name."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: CNAMERecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class CNAMERecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    flatten_cname: bool
    """
    If enabled, causes the CNAME record to be resolved externally and the resulting
    address records (e.g., A and AAAA) to be returned instead of the CNAME record
    itself. This setting is unavailable for proxied records, since they are always
    flattened.
    """

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class MXRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["MX"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """A valid mail server hostname."""

    priority: float
    """
    Required for MX and URI records; ignored for other record types (but may still
    be returned by the API). Records with lower priorities are preferred. This field
    is to be deprecated in favor of the priority field within the data map.
    """

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: MXRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class MXRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class NSRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["NS"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """A valid name server host name."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: NSRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class NSRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class DNSRecordsOpenpgpkeyRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["OPENPGPKEY"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """A single Base64-encoded OpenPGP Transferable Public Key (RFC 4880 Section 11.1)"""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: DNSRecordsOpenpgpkeyRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class DNSRecordsOpenpgpkeyRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class PTRRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["PTR"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """Domain name pointing to the address."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: PTRRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class PTRRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class TXTRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["TXT"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """Text content for the record.

    The content must consist of quoted "character strings" (RFC 1035), each with a
    length of up to 255 bytes. Strings exceeding this allowed maximum length are
    automatically split.

    Learn more at
    <https://www.cloudflare.com/learning/dns/dns-records/dns-txt-record/>.
    """

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: TXTRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class TXTRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class CAARecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["CAA"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: CAARecordData
    """Components of a CAA record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: CAARecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class CAARecordData(TypedDict, total=False):
    """Components of a CAA record."""

    flags: float
    """Flags for the CAA record."""

    tag: str
    """Name of the property controlled by this record (e.g.: issue, issuewild, iodef)."""

    value: str
    """Value of the record. This field's semantics depend on the chosen tag."""


class CAARecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class CERTRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["CERT"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: CERTRecordData
    """Components of a CERT record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: CERTRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class CERTRecordData(TypedDict, total=False):
    """Components of a CERT record."""

    algorithm: float
    """Algorithm."""

    certificate: str
    """Certificate."""

    key_tag: float
    """Key Tag."""

    type: float
    """Type."""


class CERTRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class DNSKEYRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["DNSKEY"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: DNSKEYRecordData
    """Components of a DNSKEY record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: DNSKEYRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class DNSKEYRecordData(TypedDict, total=False):
    """Components of a DNSKEY record."""

    algorithm: float
    """Algorithm."""

    flags: float
    """Flags."""

    protocol: float
    """Protocol."""

    public_key: str
    """Public Key."""


class DNSKEYRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class DSRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["DS"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: DSRecordData
    """Components of a DS record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: DSRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class DSRecordData(TypedDict, total=False):
    """Components of a DS record."""

    algorithm: float
    """Algorithm."""

    digest: str
    """Digest."""

    digest_type: float
    """Digest Type."""

    key_tag: float
    """Key Tag."""


class DSRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class HTTPSRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["HTTPS"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: HTTPSRecordData
    """Components of a HTTPS record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: HTTPSRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class HTTPSRecordData(TypedDict, total=False):
    """Components of a HTTPS record."""

    priority: float
    """Priority."""

    target: str
    """Target."""

    value: str
    """Value."""


class HTTPSRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class LOCRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["LOC"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: LOCRecordData
    """Components of a LOC record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: LOCRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class LOCRecordData(TypedDict, total=False):
    """Components of a LOC record."""

    altitude: float
    """Altitude of location in meters."""

    lat_degrees: float
    """Degrees of latitude."""

    lat_direction: Literal["N", "S"]
    """Latitude direction."""

    lat_minutes: float
    """Minutes of latitude."""

    lat_seconds: float
    """Seconds of latitude."""

    long_degrees: float
    """Degrees of longitude."""

    long_direction: Literal["E", "W"]
    """Longitude direction."""

    long_minutes: float
    """Minutes of longitude."""

    long_seconds: float
    """Seconds of longitude."""

    precision_horz: float
    """Horizontal precision of location."""

    precision_vert: float
    """Vertical precision of location."""

    size: float
    """Size of location in meters."""


class LOCRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class NAPTRRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["NAPTR"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: NAPTRRecordData
    """Components of a NAPTR record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: NAPTRRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class NAPTRRecordData(TypedDict, total=False):
    """Components of a NAPTR record."""

    flags: str
    """Flags."""

    order: float
    """Order."""

    preference: float
    """Preference."""

    regex: str
    """Regex."""

    replacement: str
    """Replacement."""

    service: str
    """Service."""


class NAPTRRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class SMIMEARecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["SMIMEA"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: SMIMEARecordData
    """Components of a SMIMEA record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: SMIMEARecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class SMIMEARecordData(TypedDict, total=False):
    """Components of a SMIMEA record."""

    certificate: str
    """Certificate."""

    matching_type: float
    """Matching Type."""

    selector: float
    """Selector."""

    usage: float
    """Usage."""


class SMIMEARecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class SRVRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["SRV"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: SRVRecordData
    """Components of a SRV record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: SRVRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class SRVRecordData(TypedDict, total=False):
    """Components of a SRV record."""

    port: float
    """The port of the service."""

    priority: float
    """
    Required for MX and URI records; ignored for other record types (but may still
    be returned by the API). Records with lower priorities are preferred. This field
    is to be deprecated in favor of the priority field within the data map.
    """

    target: str
    """A valid hostname."""

    weight: float
    """The record weight."""


class SRVRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class SSHFPRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["SSHFP"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: SSHFPRecordData
    """Components of a SSHFP record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: SSHFPRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class SSHFPRecordData(TypedDict, total=False):
    """Components of a SSHFP record."""

    algorithm: float
    """Algorithm."""

    fingerprint: str
    """Fingerprint."""

    type: float
    """Type."""


class SSHFPRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class SVCBRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["SVCB"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: SVCBRecordData
    """Components of a SVCB record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: SVCBRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class SVCBRecordData(TypedDict, total=False):
    """Components of a SVCB record."""

    priority: float
    """Priority."""

    target: str
    """Target."""

    value: str
    """Value."""


class SVCBRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class TLSARecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["TLSA"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: TLSARecordData
    """Components of a TLSA record."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: TLSARecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class TLSARecordData(TypedDict, total=False):
    """Components of a TLSA record."""

    certificate: str
    """Certificate."""

    matching_type: float
    """Matching Type."""

    selector: float
    """Selector."""

    usage: float
    """Usage."""


class TLSARecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class URIRecord(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["URI"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    data: URIRecordData
    """Components of a URI record."""

    priority: float
    """
    Required for MX and URI records; ignored for other record types (but may still
    be returned by the API). Records with lower priorities are preferred. This field
    is to be deprecated in favor of the priority field within the data map.
    """

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: URIRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class URIRecordData(TypedDict, total=False):
    """Components of a URI record."""

    target: str
    """The record content."""

    weight: float
    """The record weight."""


class URIRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


RecordUpdateParams: TypeAlias = Union[
    ARecord,
    AAAARecord,
    CNAMERecord,
    MXRecord,
    NSRecord,
    DNSRecordsOpenpgpkeyRecord,
    PTRRecord,
    TXTRecord,
    CAARecord,
    CERTRecord,
    DNSKEYRecord,
    DSRecord,
    HTTPSRecord,
    LOCRecord,
    NAPTRRecord,
    SMIMEARecord,
    SRVRecord,
    SSHFPRecord,
    SVCBRecord,
    TLSARecord,
    URIRecord,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ListCursor"]


class ListCursor(BaseModel):
    after: Optional[str] = None

    before: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ModelListParams"]


class ModelListParams(TypedDict, total=False):
    account_id: Required[str]

    author: str
    """Filter by Author"""

    format: Literal["openrouter"]
    """
    If set, return models in the requested marketplace format instead of the default
    response.
    """

    hide_experimental: bool
    """Filter to hide experimental models"""

    page: int

    per_page: int

    search: str
    """Search"""

    source: float
    """Filter by Source Id"""

    task: str
    """Filter by Task Name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.fraud import fraud_update_params
from ..._base_client import make_request_options
from ...types.fraud.fraud_settings import FraudSettings

__all__ = ["FraudResource", "AsyncFraudResource"]


class FraudResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FraudResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FraudResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FraudResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FraudResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        zone_id: str,
        authentication_settings: fraud_update_params.AuthenticationSettings | Omit = omit,
        user_profiles: Literal["enabled", "disabled"] | Omit = omit,
        username_expressions: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FraudSettings]:
        """
        Update Fraud Detection settings for a zone.

        Notes on `username_expressions` behavior:

        - If omitted or set to null, expressions are not modified.
        - If provided as an empty array `[]`, all expressions will be cleared.

        Args:
          zone_id: Identifier.

          authentication_settings: Configuration for classifying login authentication outcomes based on the origin
              response. Requires `user_profiles` to be enabled.

              - Success and failure criteria are independently updatable — sending only
                `success_criteria` leaves failure codes untouched, and vice versa.
              - Omit `authentication_settings` entirely to leave both unchanged.
              - Status codes must not overlap between success and failure criteria.

          user_profiles: Whether Fraud User Profiles is enabled for the zone.

          username_expressions: List of expressions to detect usernames in write HTTP requests.

              - Maximum of 10 expressions.
              - Omit or set to null to leave unchanged on update.
              - Provide an empty array `[]` to clear all expressions on update.
              - Invalid expressions will result in a 10400 Bad Request with details in the
                `messages` array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/fraud_detection/settings", zone_id=zone_id),
            body=maybe_transform(
                {
                    "authentication_settings": authentication_settings,
                    "user_profiles": user_profiles,
                    "username_expressions": username_expressions,
                },
                fraud_update_params.FraudUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FraudSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FraudSettings]], ResultWrapper[FraudSettings]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FraudSettings]:
        """
        Retrieve Fraud Detection settings for a zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/fraud_detection/settings", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FraudSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FraudSettings]], ResultWrapper[FraudSettings]),
        )


class AsyncFraudResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFraudResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFraudResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFraudResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFraudResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        zone_id: str,
        authentication_settings: fraud_update_params.AuthenticationSettings | Omit = omit,
        user_profiles: Literal["enabled", "disabled"] | Omit = omit,
        username_expressions: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FraudSettings]:
        """
        Update Fraud Detection settings for a zone.

        Notes on `username_expressions` behavior:

        - If omitted or set to null, expressions are not modified.
        - If provided as an empty array `[]`, all expressions will be cleared.

        Args:
          zone_id: Identifier.

          authentication_settings: Configuration for classifying login authentication outcomes based on the origin
              response. Requires `user_profiles` to be enabled.

              - Success and failure criteria are independently updatable — sending only
                `success_criteria` leaves failure codes untouched, and vice versa.
              - Omit `authentication_settings` entirely to leave both unchanged.
              - Status codes must not overlap between success and failure criteria.

          user_profiles: Whether Fraud User Profiles is enabled for the zone.

          username_expressions: List of expressions to detect usernames in write HTTP requests.

              - Maximum of 10 expressions.
              - Omit or set to null to leave unchanged on update.
              - Provide an empty array `[]` to clear all expressions on update.
              - Invalid expressions will result in a 10400 Bad Request with details in the
                `messages` array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/fraud_detection/settings", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "authentication_settings": authentication_settings,
                    "user_profiles": user_profiles,
                    "username_expressions": username_expressions,
                },
                fraud_update_params.FraudUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FraudSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FraudSettings]], ResultWrapper[FraudSettings]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FraudSettings]:
        """
        Retrieve Fraud Detection settings for a zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/fraud_detection/settings", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FraudSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FraudSettings]], ResultWrapper[FraudSettings]),
        )


class FraudResourceWithRawResponse:
    def __init__(self, fraud: FraudResource) -> None:
        self._fraud = fraud

        self.update = to_raw_response_wrapper(
            fraud.update,
        )
        self.get = to_raw_response_wrapper(
            fraud.get,
        )


class AsyncFraudResourceWithRawResponse:
    def __init__(self, fraud: AsyncFraudResource) -> None:
        self._fraud = fraud

        self.update = async_to_raw_response_wrapper(
            fraud.update,
        )
        self.get = async_to_raw_response_wrapper(
            fraud.get,
        )


class FraudResourceWithStreamingResponse:
    def __init__(self, fraud: FraudResource) -> None:
        self._fraud = fraud

        self.update = to_streamed_response_wrapper(
            fraud.update,
        )
        self.get = to_streamed_response_wrapper(
            fraud.get,
        )


class AsyncFraudResourceWithStreamingResponse:
    def __init__(self, fraud: AsyncFraudResource) -> None:
        self._fraud = fraud

        self.update = async_to_streamed_response_wrapper(
            fraud.update,
        )
        self.get = async_to_streamed_response_wrapper(
            fraud.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CredentialCreateParams"]


class CredentialCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    location: Required[Literal["header", "cookie"]]
    """Where the credential is attached in outgoing requests."""

    location_name: Required[str]
    """Name of the header or cookie where the credential is attached."""

    name: Required[str]
    """Human-readable name."""

    value: Required[str]
    """The credential value (e.g.

    API key, session token). Write-only. Never returned in responses.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["BlockPageSettingsParam"]


class BlockPageSettingsParam(TypedDict, total=False):
    """Specify block page layout settings."""

    background_color: str
    """
    Specify the block page background color in `#rrggbb` format when the mode is
    customized_block_page.
    """

    enabled: Optional[bool]
    """Specify whether to enable the custom block page."""

    footer_text: str
    """Specify the block page footer text when the mode is customized_block_page."""

    header_text: str
    """Specify the block page header text when the mode is customized_block_page."""

    include_context: bool
    """Specify whether to append context to target_uri as query parameters.

    This applies only when the mode is redirect_uri.
    """

    logo_path: str
    """Specify the full URL to the logo file when the mode is customized_block_page."""

    mailto_address: str
    """
    Specify the admin email for users to contact when the mode is
    customized_block_page.
    """

    mailto_subject: str
    """
    Specify the subject line for emails created from the block page when the mode is
    customized_block_page.
    """

    mode: Literal["", "customized_block_page", "redirect_uri"]
    """
    Specify whether to redirect users to a Cloudflare-hosted block page or a
    customer-provided URI.
    """

    name: str
    """Specify the block page title when the mode is customized_block_page."""

    suppress_footer: bool
    """
    Specify whether to suppress detailed information at the bottom of the block page
    when the mode is customized_block_page.
    """

    target_uri: str
    """Specify the URI to redirect users to when the mode is redirect_uri."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ip import IP as IP
from .dns import DNS as DNS
from .domain import Domain as Domain
from .sinkhole import Sinkhole as Sinkhole
from .ip_get_params import IPGetParams as IPGetParams
from .domain_history import DomainHistory as DomainHistory
from .dns_list_params import DNSListParams as DNSListParams
from .ip_get_response import IPGetResponse as IPGetResponse
from .whois_get_params import WhoisGetParams as WhoisGetParams
from .domain_get_params import DomainGetParams as DomainGetParams
from .whois_get_response import WhoisGetResponse as WhoisGetResponse
from .domain_history_get_params import DomainHistoryGetParams as DomainHistoryGetParams
from .domain_history_get_response import DomainHistoryGetResponse as DomainHistoryGetResponse
from .indicator_feed_get_response import IndicatorFeedGetResponse as IndicatorFeedGetResponse
from .indicator_feed_create_params import IndicatorFeedCreateParams as IndicatorFeedCreateParams
from .indicator_feed_data_response import IndicatorFeedDataResponse as IndicatorFeedDataResponse
from .indicator_feed_list_response import IndicatorFeedListResponse as IndicatorFeedListResponse
from .indicator_feed_update_params import IndicatorFeedUpdateParams as IndicatorFeedUpdateParams
from .indicator_feed_create_response import IndicatorFeedCreateResponse as IndicatorFeedCreateResponse
from .indicator_feed_update_response import IndicatorFeedUpdateResponse as IndicatorFeedUpdateResponse
from .miscategorization_create_params import MiscategorizationCreateParams as MiscategorizationCreateParams
from .miscategorization_create_response import MiscategorizationCreateResponse as MiscategorizationCreateResponse
